@@ -69,6 +69,7 @@
 #include <sys/time.h>
 #include <regex.h>
 #include <math.h>
+#include <float.h>
 #include <errno.h>
 
 /* δؿѤǡ*/
@@ -941,7 +942,7 @@ static MssValue cal_fix(struct mssCal *cal)
       *(v.v.s+pos)=pad;
       pos--;
     }
-    *(v.v.s+pos)='\0';
+    *(v.v.s+len)='\0';
     break;
   default:
     mssShowErrMsg("specify [R] or [L] in the third parameter in [fix] function.");
@@ -1385,28 +1386,16 @@ static MssValue cal_abs(struct mssCal *cal)
 static MssValue cal_up(struct mssCal *cal)
 {
   MssValue v;
-  double v0;
-  double v1;
-  double from;
-  double to;
   mssVinit(&v,DBL);
 
   if(cal->arg[0].val[0].nul ||
      cal->arg[1].val[0].nul ||
-     cal->arg[1].val[0].v.d==0    ){
+     cal->arg[1].val[0].v.d<=0 ){
     MssVnull(v);
     return(v);
   }
 
-  v0=cal->arg[0].val[0].v.d;
-  v1=cal->arg[1].val[0].v.d;
-
-  from=(double)((int)(v0/v1)  )*v1;
-  if(v0>0) to  =(double)((int)(v0/v1)+1)*v1;
-  else     to  =(double)((int)(v0/v1)-1)*v1;
-
-  if(v0==from) v.v.d=from;
-  else         v.v.d=to;
+  v.v.d=ceil(cal->arg[0].val[0].v.d/cal->arg[1].val[0].v.d)*cal->arg[1].val[0].v.d;
 
   return(v);
 }
@@ -1427,36 +1416,29 @@ static MssValue cal_up(struct mssCal *cal)
 static MssValue cal_down(struct mssCal *cal)
 {
   MssValue v;
-  double v0;
-  double v1;
-  double from;
-  double to;
+  double diff;
   mssVinit(&v,DBL);
 
   if(cal->arg[0].val[0].nul ||
      cal->arg[1].val[0].nul ||
-     cal->arg[1].val[0].v.d==0    ){
+     cal->arg[1].val[0].v.d<=0 ){
     MssVnull(v);
     return(v);
   }
 
-  v0=cal->arg[0].val[0].v.d;
-  v1=cal->arg[1].val[0].v.d;
-
-  from=(double)((int)(v0/v1)  )*v1;
-  if(v0>0) to  =(double)((int)(v0/v1)+1)*v1;
-  else     to  =(double)((int)(v0/v1)-1)*v1;
-
-  if(v0==to) v.v.d=to;
-  else       v.v.d=from;
+  v.v.d=ceil(cal->arg[0].val[0].v.d/cal->arg[1].val[0].v.d)*cal->arg[1].val[0].v.d;
+  diff=v.v.d - cal->arg[0].val[0].v.d;
+  if(diff > DBL0 || diff < -DBL0 ){
+    v.v.d -= cal->arg[1].val[0].v.d;
+  }
 
   return(v);
 }
 
 /**
  * # FUNCTION #
- * upؿ
- * Ϳ up(,)
+ * roundؿ
+ * Ϳ round(,)
  * ͤͤˤäƻͼθ֤ͤ
  * ͤˤϡͼθ̿ͤñ̤ꤹ롣
  * ΤˤСͼθǤϤʤͤܤοͽǺǤᤤ
@@ -1473,40 +1455,59 @@ static MssValue cal_round(struct mssCal *cal)
   MssValue v;
   double v0;
   double v1;
-  double from;
-  double to;
-  double center;
+  double vint;
   mssVinit(&v,DBL);
 
   if(cal->arg[0].val[0].nul ||
      cal->arg[1].val[0].nul ||
-     cal->arg[1].val[0].v.d==0    ){
+     cal->arg[1].val[0].v.d<=0 ){
     MssVnull(v);
     return(v);
   }
 
   v0=cal->arg[0].val[0].v.d;
   v1=cal->arg[1].val[0].v.d;
-
-  if(v0>0){
-    from=(double)((int)(v0/v1)  )*v1;
-    to  =(double)((int)(v0/v1)+1)*v1;
-    center=(from+to)/2;
-
-    if(v0<center) v.v.d=from;
-    else          v.v.d=to;
-  }else{
-    from=(double)((int)(v0/v1)  )*v1;
-    to  =(double)((int)(v0/v1)-1)*v1;
-    center=(from+to)/2;
-
-    if(v0>center) v.v.d=from;
-    else          v.v.d=to;
-  }
+  if(v0<0) modf((v0-(v1/2)+DBL0)/v1,&vint);
+  else     modf((v0+(v1/2)-DBL0)/v1,&vint);
+  v.v.d=vint*v1;
 
   return(v);
 }
 
+/**
+ * # FUNCTION #
+ * fixdecؿ
+ * Ϳ up(,ʲ)
+ * ͤξʲη򡢻ꤷͤǤʸȤ֤
+ */
+static MssValue cal_fixdec(struct mssCal *cal)
+{
+  char buf[256];
+  char fmt[20];
+  MssValue v;
+  mssVinit(&v,STR);
+
+  if(cal->arg[0].val[0].nul ||
+     cal->arg[1].val[0].nul ||
+     cal->arg[1].val[0].v.d<0 ||
+     cal->arg[1].val[0].v.d>6 ){
+    MssVnull(v);
+    return(v);
+  }
+
+  fmt[0]='%';
+  fmt[1]='.';
+  sprintf(&fmt[2],"%df",(int)cal->arg[1].val[0].v.d);
+
+  sprintf(buf,fmt,cal->arg[0].val[0].v.d);
+
+  v.v.s=mssStrdup(buf);
+
+  setStrMalAdd(v.v.s);
+  return(v);
+}
+
+
 /**
  * # FUNCTION #
  * ͭդʤ飱򤽤ǤʤУ֤
@@ -2106,7 +2107,7 @@ static MssValue cal_day(struct mssCal *cal)
 
   mssVinit(&v,DBL);
 
-  if(cal->arg[0].val[0].nul){
+  if(cal->arg[0].val[0].nul || cal->arg[1].val[0].nul){
     MssVnull(v);
     return(v);
   }
@@ -2146,7 +2147,7 @@ static MssValue cal_time(struct mssCal *cal)
   char *v1;
   mssVinit(&v,DBL);
 
-  if(cal->arg[0].val[0].nul){
+  if(cal->arg[0].val[0].nul || cal->arg[1].val[0].nul){
     MssVnull(v);
     return(v);
   }
@@ -2187,7 +2188,7 @@ static MssValue cal_time(struct mssCal *cal)
     MssVnull(v);
     return(v);
   }
-  v.v.d = (double)difftime(tSec,fSec);
+  v.v.d = (double)difftime(fSec,tSec);
   return(v);
 }
 
@@ -2711,6 +2712,7 @@ struct Func func[]={
   {"up"         ,0, 1,{N,N,E}  ,N   ,7 ,cal_up},
   {"down"       ,0, 1,{N,N,E}  ,N   ,7 ,cal_down},
   {"round"      ,0, 1,{N,N,E}  ,N   ,7 ,cal_round},
+  {"fixdec"     ,0, 1,{N,N,E}  ,S   ,7 ,cal_fixdec},
   {"pi"         ,0, 1,{E}      ,N   ,7 ,cal_pi},
   {"exp"        ,0, 1,{N,E}    ,N   ,7 ,cal_Exp},
   {"ln"         ,0, 1,{N,E}    ,N   ,7 ,cal_ln},
