@@ -46,6 +46,7 @@ struct mssFldInfo *mssInitFldInfo()
   fi->revFlg=0;
   fi->numFlg=0;
   fi->length=0;
+  fi->comment=NULL;
 
   return(fi);
 }
@@ -59,6 +60,7 @@ void mssFreeFldInfo( struct mssFldInfo *fi)
 
   if(fi!=NULL){
     mssFree(fi->name);
+    mssFree(fi->comment);
   }
   mssFree(fi);
 }
@@ -69,8 +71,29 @@ void mssFreeFldInfo( struct mssFldInfo *fi)
  */
 void mssShowFldInfo(struct mssFldInfo *fi)
 {
-  printf("\tnum=%d name=%s priority=%d revFlg=%d numFlg=%d\n",
-       fi->num, fi->name, fi->priority, fi->revFlg, fi->numFlg);
+  printf("\tnum=%d name=%s priority=%d revFlg=%d numFlg=%d comment=%s\n",
+       fi->num, fi->name, fi->priority, fi->revFlg, fi->numFlg, fi->comment);
+}
+
+/**
+ * # FUNCTION #
+ * numǻꤵ줿ֹιܤmssFields¤ΤmssFldInfo¤(flds->fi)
+ * õĤФΥݥ󥿤֤
+ * ĤʤХ顼ɽƽλ롣
+ * ǻꤵnumǤιֹǡϤޤ롣
+ */
+struct mssFldInfo *mssFldNum2Add(struct mssFields *flds, int num)
+{
+  int i;
+
+  for(i=0; i<flds->cnt; i++){
+    if( num==MssFlds2num(flds,i) ){
+      return( *(flds->fi+i) );
+    }
+  }
+  mssShowErrMsg("internal error field number not found : [%d]",num);
+  mssEnd(mssErrorNoDefault);
+  return(NULL); /*ѥٹ򤱤뤿*/
 }
 
 /**
@@ -246,7 +269,6 @@ void mssFreeHeader(struct mssHeader *hd)
   mssFreeFields(hd->flds);
   mssFree(hd->xmlver);
   mssFree(hd->xmlenc);
-  mssFree(hd->version);
   mssFree(hd);
 }
 
@@ -266,7 +288,7 @@ struct mssHeader *mssInitHeader(char *title, char *comment)
   hd->comment= mssStrdup(comment);
   hd->xmlver = mssStrdup(MssXmlDefVer);
   hd->xmlenc = mssStrdup(MssXmlDefEnc);
-  hd->version= mssStrdup(MssXtDefVer);
+  hd->version= MssXtDefVer;
   hd->flds   = mssInitFields();
   hd->recCnt = -1;
 
@@ -277,7 +299,8 @@ struct mssHeader *mssInitHeader(char *title, char *comment)
  * # FUNCTION #
  * mssHeader¤Τoldǻꤷ¸Υإå(ܾϽ)
  * ԡ롣
- * ԡϡtitle,comment,xmlver,xmlenc,versionǡ¾Ͻ롣
+ * ԡϡtitle,comment,xmlver,xmlencǡ¾Ͻ롣
+ * version MssXtDefVerǽ롣
  * ԡκݤmssStrdupѤΤǡ
  * ¸Υإå(ʸ)ݥ󥿤Ƕͭ뤳ȤϤʤ
  */
@@ -299,11 +322,9 @@ struct mssHeader *mssInitCpyHeader(struct mssHeader *old)
   if(NULL!=old->xmlenc)  hd->xmlenc = mssStrdup(old->xmlenc);
   else                   hd->xmlenc = NULL;
 
-  if(NULL!=old->version) hd->version= mssStrdup(old->version);
-  else                   hd->version= NULL;
-
+  hd->version = MssXtDefVer;
   hd->flds    = mssInitFields();
-  hd->recCnt = -1;
+  hd->recCnt  = -1;
 
   return(hd);
 }
@@ -315,7 +336,7 @@ struct mssHeader *mssInitCpyHeader(struct mssHeader *old)
  * κݤmssStrdupѤΤǡ
  * ʸݥ󥿤Ƕͭ뤳ȤϤʤ
  */
-struct mssHeader *mssInitSetHeader( char *title, char *comment, char *xmlver, char *xmlenc, char *version)
+struct mssHeader *mssInitSetHeader( char *title, char *comment, char *xmlver, char *xmlenc, int version)
 {
   struct mssHeader *hd;
 
@@ -333,8 +354,7 @@ struct mssHeader *mssInitSetHeader( char *title, char *comment, char *xmlver, ch
   if(NULL!=xmlenc)  hd->xmlenc = mssStrdup(xmlenc);
   else              hd->xmlenc = NULL;
 
-  if(NULL!=version) hd->version= mssStrdup(version);
-  else              hd->version= NULL;
+  hd->version= version;
 
   hd->flds    = mssInitFields();
   hd->recCnt = -1;
@@ -505,7 +525,7 @@ void mssSetFieldsSort(struct mssFields *flds, struct mssFields *sortFlds)
 
   for(i=0; i<sortFlds->cnt; i++){
     no = MssFlds2num(sortFlds,i);
-    MssFlds2priority(flds,no)= MssFlds2priority(sortFlds,i);
+    MssFlds2priority(flds,no)= i+1; //MssFlds2priority(sortFlds,i);
     MssFlds2revFlg(flds,no)  = MssFlds2revFlg(sortFlds,i);
     MssFlds2numFlg(flds,no)  = MssFlds2numFlg(sortFlds,i);
   }
@@ -536,14 +556,31 @@ void mssSetFldInfoLength(struct mssFldInfo *fi, int length)
 
 /**
  * # FUNCTION #
- * mssFldInfo¤Τι̾ȹֹ򡢻ι̾(name)ֹ(num)
- * ˤäƥåȤ롣
+ * mssFldInfo¤ΤιֹnumˤäƥåȤ롣
+ * ǡǤ1ϤޤֹŪˤϣϤޤֹ档
+ * ǤǤϡnumˤֹȤͿ뤳ȡ
  */
-void mssSetFldInfoName(struct mssFldInfo *fi, char *name, int num)
+void mssSetFldInfoNum(struct mssFldInfo *fi, int num)
 {
   fi->num      = num;
+}
+
+/**
+ * # FUNCTION #
+ * mssFldInfo¤Τι̾name ˤäƥåȤ롣
+ */
+void mssSetFldInfoName(struct mssFldInfo *fi, char *name)
+{
   fi->name     = mssStrdup(name);
-/*  fi->comment  = NULL; */
+}
+
+/**
+ * # FUNCTION #
+ * mssFldInfo¤ΤΥȤcommentˤäƥåȤ롣
+ */
+void mssSetFldInfoComment(struct mssFldInfo *fi, char *comment)
+{
+  fi->comment = mssStrdup(comment);
 }
 
 /**
@@ -570,9 +607,12 @@ void mssAddFieldsByStr(struct mssFields *flds, char *str)
   flds->fi=mssRealloc(flds->fi,sizeof(struct mssFldInfo *)*(flds->cnt+1),
                         "AddFieldsByStr");
   *(flds->fi+flds->cnt) = mssInitFldInfo();
-  mssSetFldInfoName(*(flds->fi+flds->cnt),str,flds->cnt);
+
+  mssSetFldInfoNum(*(flds->fi+flds->cnt),flds->cnt);
+  mssSetFldInfoName(*(flds->fi+flds->cnt),str);
   mssSetFldInfoSort(*(flds->fi+flds->cnt),0,0,0);
   mssSetFldInfoLength(*(flds->fi+flds->cnt),0);
+  mssSetFldInfoComment(*(flds->fi+flds->cnt),NULL);
 
   flds->cnt++;
 }
@@ -604,9 +644,11 @@ void mssAddFieldsByFldInfo(struct mssFields *flds, struct mssFldInfo *fi)
   *(flds->fi+flds->cnt) = mssInitFldInfo();
   ofi=*(flds->fi+flds->cnt);
 
-  mssSetFldInfoName(ofi,fi->name,fi->num);
+  mssSetFldInfoNum(ofi,fi->num);
+  mssSetFldInfoName(ofi,fi->name);
   mssSetFldInfoSort(ofi,fi->priority,fi->revFlg,fi->numFlg);
   mssSetFldInfoLength(ofi,fi->length);
+  mssSetFldInfoComment(ofi,fi->comment);
 
   flds->cnt++;
 }
@@ -659,9 +701,11 @@ void mssAddHeadOrOptFields(struct mssFields *flds, struct mssHeader *hd, MssOptF
     /*ץǻꤵʤä*/
     num=*(optFld->fldNo2optNo+i);
     if(*(optFld->fldNo2optNo+i)!=-1){
-      mssSetFldInfoName(*(flds->fi+flds->cnt-1), *(optFld->newNam+num), i);
+      mssSetFldInfoNum(*(flds->fi+flds->cnt-1), i);
+      mssSetFldInfoName(*(flds->fi+flds->cnt-1), *(optFld->newNam+num));
       mssSetFldInfoSort(*(flds->fi+flds->cnt-1),0,0,0);
       mssSetFldInfoLength(*(flds->fi+flds->cnt-1),0);
+      mssSetFldInfoComment(*(flds->fi+flds->cnt-1), NULL);
     }
   }
 }
@@ -743,7 +787,7 @@ static void chkOtfFldName(struct mssHeader *hd, struct mssFPW *fp )
  * # FUNCTION #
  * mssFldInfo¤ξإåȤƽ񤭽Ф
  */
-void mssWriteFldInfo( struct mssFldInfo *fi, struct mssFPW *fp)
+void mssWriteFldInfo( struct mssFldInfo *fi, int version, struct mssFPW *fp)
 {
   struct mssXmlTag *fieldTag;
   struct mssXmlTag *sortTag;
@@ -751,48 +795,83 @@ void mssWriteFldInfo( struct mssFldInfo *fi, struct mssFPW *fp)
   struct mssXmlTag *numTag;
   char *str;
 
-  /* <field no="1"> */
-  fieldTag=mssInitXmlTag("field",NULL);
-  mssAddXmlTagAttributeInt(fieldTag,"no",fi->num+1,NULL);
-  str=mssXmlTag2startTag(fieldTag, NULL);
-  mssWriteStr(str,fp); mssFree(str);
-
-  /* <name>fieldName</name> */
-  mssWriteXmlTagStr(0,"name",fi->name,0,NULL,fp);
-
-  /* <sort priority="1"><numeric/><reverse/></sort>*/
-  if(0 != fi->priority){
-    sortTag=mssInitXmlTag("sort",NULL);
-    mssAddXmlTagAttributeInt(sortTag,"priority",fi->priority,NULL);
-    str=mssXmlTag2startTag(sortTag, NULL);
+  switch(version){
+  /*XMLtable version 1.0*/
+  case 10:
+    /* <field no="1"> */
+    fieldTag=mssInitXmlTag("field",NULL);
+    mssAddXmlTagAttributeInt(fieldTag,"no",fi->num+1,NULL);
+    str=mssXmlTag2startTag(fieldTag, NULL);
     mssWriteStr(str,fp); mssFree(str);
-    if(fi->revFlg){
-      revTag=mssInitXmlTag("reverse",NULL);
-      str=mssXmlTag2emptyTag(revTag, NULL);
+
+    /* <name>fieldName</name> */
+    mssWriteXmlTagStr(0,"name",fi->name,0,NULL,fp);
+
+    /* <sort priority="1"><numeric/><reverse/></sort>*/
+    if(0 != fi->priority){
+      sortTag=mssInitXmlTag("sort",NULL);
+      mssAddXmlTagAttributeInt(sortTag,"priority",fi->priority,NULL);
+      str=mssXmlTag2startTag(sortTag, NULL);
       mssWriteStr(str,fp); mssFree(str);
-      mssFreeXmlTag(revTag);
-    }
-    if(fi->numFlg){
-      numTag=mssInitXmlTag("numeric",NULL);
-      str=mssXmlTag2emptyTag(numTag, NULL);
+      if(fi->revFlg){
+        revTag=mssInitXmlTag("reverse",NULL);
+        str=mssXmlTag2emptyTag(revTag, NULL);
+        mssWriteStr(str,fp); mssFree(str);
+        mssFreeXmlTag(revTag);
+      }
+      if(fi->numFlg){
+        numTag=mssInitXmlTag("numeric",NULL);
+        str=mssXmlTag2emptyTag(numTag, NULL);
+        mssWriteStr(str,fp); mssFree(str);
+        mssFreeXmlTag(numTag);
+      }
+      str=mssXmlTag2endTag(sortTag, NULL);
       mssWriteStr(str,fp); mssFree(str);
-      mssFreeXmlTag(numTag);
+      mssFreeXmlTag(sortTag);
     }
-    str=mssXmlTag2endTag(sortTag, NULL);
+
+    /*<length>10</length>*/
+    if(fi->length!=0){
+      mssWriteXmlTagInt(0,"length",fi->length,0,NULL,fp);
+    }
+
+    /*</field>*/
+    str=mssXmlTag2endTag(fieldTag, NULL);
     mssWriteStr(str,fp); mssFree(str);
-    mssFreeXmlTag(sortTag);
-  }
+    mssFreeXmlTag(fieldTag);
+    mssWriteRet(fp);
+    break;
 
-  /*<length>10</length>*/
-  if(fi->length!=0){
-    mssWriteXmlTagInt(0,"length",fi->length,0,NULL,fp);
-  }
+    /*XMLtable version 1.1*/
+    /*<field num="1" name="̾" sort="1" numeric="1" reverse="1" length="10">ܥ<field> */
+  case 11:
+    fieldTag=mssInitXmlTag("field",NULL);
+    mssAddXmlTagAttributeInt(fieldTag,"no"  ,fi->num+1,NULL);
+    mssAddXmlTagAttributeStr(fieldTag,"name",fi->name ,NULL);
+    if(0 != fi->priority){
+      mssAddXmlTagAttributeInt(fieldTag,"sort",fi->priority ,NULL);
+      if(fi->numFlg){
+        mssAddXmlTagAttributeInt(fieldTag,"numeric",1 ,NULL);
+      }
+      if(fi->revFlg){
+        mssAddXmlTagAttributeInt(fieldTag,"reverse",1 ,NULL);
+      }
+    }
+    if(fi->length!=0){
+      mssAddXmlTagAttributeInt(fieldTag,"length",fi->length ,NULL);
+    }
+    mssWriteXmlStartTag(fieldTag, NULL, fp);
+    if(fi->comment!=NULL){
+      mssWriteStr(fi->comment,fp);
+    }
+    mssWriteXmlEndTag(fieldTag, NULL, fp);
+    mssWriteRet(fp);
+    break;
 
-  /*</field>*/
-  str=mssXmlTag2endTag(fieldTag, NULL);
-  mssWriteStr(str,fp); mssFree(str);
-  mssFreeXmlTag(fieldTag);
-  mssWriteRet(fp);
+  default:
+    mssShowErrMsg("Internal Error: invalid XMLtable version");
+    mssEnd(mssErrorNoDefault);
+  }
 }
 
 /**
@@ -804,16 +883,30 @@ void mssWriteHeader( struct mssHeader *hd, struct mssFPW *fp)
   struct mssXmlTag *xtTag;
   struct mssXmlTag *hdTag;
   int i;
+  char *version=NULL;
+  char verStr[2][4]={"1.0","1.1"};
 
   if(mssGV.txtFlg)return;
 
+  switch(hd->version){
+  case 10:
+    version=verStr[0];
+    break;
+  case 11:
+    version=verStr[1];
+    break;
+  default:
+    mssShowErrMsg("Internal Error: invalid XMLtable version");
+    mssEnd(mssErrorNoDefault);
+  }
+
   chkOtfFldName(hd,fp); /*Ϲ̾Υå*/
 
   mssWriteXmlDeclaration( hd->xmlver, hd->xmlenc ,fp);
 
-  /*xmlTableإåν񤭽Ф<xmltbl version="1.00">*/
+  /*xmlTableإåν񤭽Ф<xmltbl version="1.1">*/
   xtTag=mssInitXmlTag("xmltbl",NULL);
-  mssAddXmlTagAttributeStr(xtTag,"version",hd->version,NULL);
+  mssAddXmlTagAttributeStr(xtTag,"version",version,NULL);
   mssWriteXmlStartTag(xtTag,NULL,fp);
   mssFreeXmlTag(xtTag);
   mssWriteRet(fp);
@@ -831,7 +924,7 @@ void mssWriteHeader( struct mssHeader *hd, struct mssFPW *fp)
 
   /*<field>*/
   for(i=0; i<hd->flds->cnt; i++){    /*إåιܥ*/
-    mssWriteFldInfo(*(hd->flds->fi+i), fp);
+    mssWriteFldInfo(*(hd->flds->fi+i), hd->version, fp);
   }
 
   /*<recCnt>*/
@@ -882,7 +975,7 @@ void mssCreateBlankXtFile(char **fldName, int fldCnt, char *fname)
   hd->comment= NULL;
   hd->xmlver = mssStrdup(MssXmlDefVer);
   hd->xmlenc = mssStrdup(MssXmlDefEnc);
-  hd->version= mssStrdup(MssXtDefVer);
+  hd->version= MssXtDefVer;
   hd->flds   = mssInitFields();
   hd->recCnt = -1;
 
@@ -920,6 +1013,7 @@ struct mssHeader *mssReadHeader( struct mssFPR *fp)
   char *str;
   int i;
   int datFldCnt;  /*ǡιܿ*/
+  int fldNum;
   int priority;
   int revFlg;
   int numFlg;
@@ -940,8 +1034,6 @@ struct mssHeader *mssReadHeader( struct mssFPR *fp)
     for(i=0; i<datFldCnt; i++){
       sprintf(numChr[i+1],"%d",i+1);
       mssAddFieldsByStr(hd->flds, mssStrdup(numChr[i+1]));
-      mssSetFldInfoSort(*(hd->flds->fi+hd->flds->cnt-1),0,0,0);
-      mssSetFldInfoLength(*(hd->flds->fi+hd->flds->cnt-1),0);
     }
   /*---------------------------- xmlTblξ*/
   }else{
@@ -997,32 +1089,126 @@ struct mssHeader *mssReadHeader( struct mssFPR *fp)
       /*оݤʸ餹Ȥǡfieldθ*/
       tmpStr=pos+1;
 
-      /*nameΥå (°ͤʤȤϤꤨʤ)*/
-      str = mssGetTagCont(field->str, "name",1);
-      mssAddFieldsByStr(hd->flds, str);
+      switch(hd->version){
+      case 10:
+        /*̾*/
+        str = mssGetTagCont(field->str, "name",1);
+        if(str!=NULL){
+          mssAddFieldsByStr(hd->flds, str);
+          mssFree(str);
+        }else{
+          mssShowErrMsg("Internal Error: invalid filed number");
+          mssEnd(mssErrorNoDefault);
+        }
 
-      /*sortΥå (Υʤ0åȤ)*/
-      priority = 0;
-      revFlg   = 0;
-      numFlg   = 0;
-      sort=mssGetTag("sort", field->str, &pos);
-      if( NULL != sort ){
-        str = mssGetTagAtt(sort->str, "sort", "priority");
-        priority = atoi(str);
-        revFlg   = mssGetNullTag(sort->str, "reverse");
-        numFlg   = mssGetNullTag(sort->str, "numeric");
+        /*ֹ*/
+        str = mssGetTagAtt(field->str, "field", "no");
+        if(str!=NULL){
+          fldNum=atoi(str);
+          if(fldNum>0 && fldNum<MssFieldMaxCnt){
+            mssSetFldInfoNum(*(hd->flds->fi+hd->flds->cnt-1), fldNum-1);
+          }else{
+            mssShowErrMsg("Internal Error: invalid filed number");
+            mssEnd(mssErrorNoDefault);
+          }
+          mssFree(str);
+        }else{
+          mssShowErrMsg("Internal Error: invalid filed number");
+          mssEnd(mssErrorNoDefault);
+        }
+
+        /*sortΥå (Υʤ0åȤ)*/
+        priority = 0;
+        revFlg   = 0;
+        numFlg   = 0;
+        sort=mssGetTag("sort", field->str, &pos);
+        if( NULL != sort ){
+          str = mssGetTagAtt(sort->str, "sort", "priority");
+          priority = atoi(str);
+          revFlg   = mssGetNullTag(sort->str, "reverse");
+          numFlg   = mssGetNullTag(sort->str, "numeric");
+          mssFree(str);
+          mssFreeStrings(sort);
+        }
+        mssSetFldInfoSort(*(hd->flds->fi+hd->flds->cnt-1),
+                                                      priority,revFlg,numFlg);
+
+        /*lenghtΥå*/
+        str = mssGetTagCont(field->str, "length",1);
+        if(str!=NULL) length = atoi(str);
+        else          length = 0;
         mssFree(str);
-        mssFreeStrings(sort);
+        mssSetFldInfoLength(*(hd->flds->fi+hd->flds->cnt-1),length);
+        break;
+
+      case 11:
+        /*̾*/
+        str = mssGetTagAtt(field->str, "field", "name");
+        if(str!=NULL){
+          mssAddFieldsByStr(hd->flds, str);
+          mssFree(str);
+        }else{
+          mssShowErrMsg("Internal Error: invalid filed number");
+          mssEnd(mssErrorNoDefault);
+        }
+
+        /*ֹ*/
+        str = mssGetTagAtt(field->str, "field", "no");
+        if(str!=NULL){
+          fldNum=atoi(str);
+          if(fldNum>0 && fldNum<MssFieldMaxCnt){
+            mssSetFldInfoNum(*(hd->flds->fi+hd->flds->cnt-1), fldNum-1);
+          }else{
+            mssShowErrMsg("Internal Error: invalid filed number");
+            mssEnd(mssErrorNoDefault);
+          }
+          mssFree(str);
+        }else{
+          mssShowErrMsg("Internal Error: invalid filed number");
+          mssEnd(mssErrorNoDefault);
+        }
+
+        /*Ⱦ*/
+        priority = 0;
+        revFlg   = 0;
+        numFlg   = 0;
+        str = mssGetTagAtt(field->str, "field", "sort");
+        if(str!=NULL){
+          priority=atoi(str);
+          mssFree(str);
+          str = mssGetTagAtt(field->str, "field", "numeric");
+          if(str!=NULL){
+            numFlg=atoi(str);
+            if(numFlg!=0 && numFlg!=1){
+              mssShowErrMsg("Internal Error: invalid numeric value");
+              mssEnd(mssErrorNoDefault);
+            }
+            mssFree(str);
+          }
+
+          str = mssGetTagAtt(field->str, "field", "reverse");
+          if(str!=NULL){
+            revFlg=atoi(str);
+            if(revFlg!=0 && revFlg!=1){
+              mssShowErrMsg("Internal Error: invalid reverse value");
+              mssEnd(mssErrorNoDefault);
+            }
+            mssFree(str);
+          }
+          mssSetFldInfoSort(*(hd->flds->fi+hd->flds->cnt-1),
+                                                      priority,revFlg,numFlg);
+        }
+
+        /**/
+        str=mssGetTagCont(field->str,"field",0);
+        mssSetFldInfoComment(*(hd->flds->fi+hd->flds->cnt-1), str);
+        mssFree(str);
+        break;
+
+      default:
+        mssShowErrMsg("Internal Error: invalid XMLtable version");
+        mssEnd(mssErrorNoDefault);
       }
-      mssSetFldInfoSort(*(hd->flds->fi+hd->flds->cnt-1),priority,revFlg,numFlg);
-
-      /*lenghtΥå*/
-      str = mssGetTagCont(field->str, "length",1);
-      if(str!=NULL) length = atoi(str);
-      else          length = 0;
-      mssFree(str);
-      mssSetFldInfoLength(*(hd->flds->fi+hd->flds->cnt-1),length);
-
       mssFreeStrings(field);
     }
     mssFreeStrings(head);
@@ -1043,7 +1229,7 @@ void mssShowHeader(struct mssHeader *hd)
   printf("====================================\n");
   printf("         Header Information         \n");
   printf("====================================\n");
-  printf("XML Table version : %s\n",hd->version);
+  printf("XML Table version : %d\n",hd->version);
   printf("title : %s\n",hd->title);
   printf("comment : %s\n",hd->comment);
   printf("fldCnt : %d\n",hd->flds->cnt);
@@ -1267,8 +1453,7 @@ void mssSetOptFld(MssOptFLD *optFLD, struct mssHeader *hd)
     if(optFLD->newFlg){
       if(0 != strcmp(*(optFLD->nam+i), *(optFLD->newNam+i)) ){
         mssSetFldInfoName(*(optFLD->flds->fi+optFLD->flds->cnt-1),
-                          *(optFLD->newNam+i),
-                          (*(optFLD->flds->fi+optFLD->flds->cnt-1))->num);
+                          *(optFLD->newNam+i));
       }
     }
 
