@@ -508,7 +508,7 @@ char *mssXmlTag2startTag(struct mssXmlTag *xmlTag, iconv_t *icid)
 
   for(i=0; i<xmlTag->attCnt/2; i++){
     buf[bc++]=' ';
-    str=*(xmlTag->attributes+i);
+    str=*(xmlTag->attributes+i*2);
     while(*str!='\0'){
       if(bc>=4093){
         mssShowErrMsg("XML tag is too long");
@@ -517,7 +517,7 @@ char *mssXmlTag2startTag(struct mssXmlTag *xmlTag, iconv_t *icid)
       buf[bc++]=*str++;
     }
     buf[bc++]='=';
-    str=*(xmlTag->attributes+i+1);
+    str=*(xmlTag->attributes+i*2+1);
     while(*str!='\0'){
       if(bc>=4093){
         mssShowErrMsg("XML tag is too long");
